@@ -30,6 +30,7 @@
 #include <QtDebug>
 #include <QSettings>
 #include <QDir>
+#include <QRegularExpression>
 
 
 // KDE
@@ -164,15 +165,15 @@ void ColorScheme::setColorTableEntry(int index , const ColorEntry& entry)
 {
     Q_ASSERT( index >= 0 && index < TABLE_COLORS );
 
-    if ( !_table ) 
+    if ( !_table )
     {
         _table = new ColorEntry[TABLE_COLORS];
 
         for (int i=0;i<TABLE_COLORS;i++)
             _table[i] = defaultTable[i];
     }
-    
-    _table[index] = entry; 
+
+    _table[index] = entry;
 }
 ColorEntry ColorScheme::colorEntry(int index , uint randomSeed) const
 {
@@ -183,19 +184,19 @@ ColorEntry ColorScheme::colorEntry(int index , uint randomSeed) const
 
     ColorEntry entry = colorTable()[index];
 
-    if ( randomSeed != 0 && 
-        _randomTable != 0 && 
+    if ( randomSeed != 0 &&
+        _randomTable != 0 &&
         !_randomTable[index].isNull() )
     {
         const RandomizationRange& range = _randomTable[index];
-      
+
 
         int hueDifference = range.hue ? (qrand() % range.hue) - range.hue/2 : 0;
         int saturationDifference = range.saturation ? (qrand() % range.saturation) - range.saturation/2 : 0;
         int  valueDifference = range.value ? (qrand() % range.value) - range.value/2 : 0;
 
         QColor& color = entry.color;
-      
+
         int newHue = qAbs( (color.hue() + hueDifference) % MAX_HUE );
         int newValue = qMin( qAbs(color.value() + valueDifference) , 255 );
         int newSaturation = qMin( qAbs(color.saturation() + saturationDifference) , 255 );
@@ -216,13 +217,13 @@ bool ColorScheme::randomizedBackgroundColor() const
 }
 void ColorScheme::setRandomizedBackgroundColor(bool randomize)
 {
-    // the hue of the background colour is allowed to be randomly 
+    // the hue of the background colour is allowed to be randomly
     // adjusted as much as possible.
     //
     // the value and saturation are left alone to maintain read-ability
     if ( randomize )
     {
-        setRandomizationRange( 1 /* background color index */ , MAX_HUE , 255 , 0 ); 
+        setRandomizationRange( 1 /* background color index */ , MAX_HUE , 255 , 0 );
     }
     else
     {
@@ -305,7 +306,7 @@ void ColorScheme::write(KConfig& config) const
 
     configGroup.writeEntry("Description",_description);
     configGroup.writeEntry("Opacity",_opacity);
-    
+
     for (int i=0 ; i < TABLE_COLORS ; i++)
     {
         RandomizationRange random = _randomTable != 0 ? _randomTable[i] : RandomizationRange();
@@ -314,13 +315,13 @@ void ColorScheme::write(KConfig& config) const
 }
 #endif
 
-QString ColorScheme::colorNameForIndex(int index) 
+QString ColorScheme::colorNameForIndex(int index)
 {
     Q_ASSERT( index >= 0 && index < TABLE_COLORS );
 
     return QString(colorNames[index]);
 }
-QString ColorScheme::translatedColorNameForIndex(int index) 
+QString ColorScheme::translatedColorNameForIndex(int index)
 {
     Q_ASSERT( index >= 0 && index < TABLE_COLORS );
 
@@ -329,21 +330,57 @@ QString ColorScheme::translatedColorNameForIndex(int index)
 
 void ColorScheme::readColorEntry(QSettings * s , int index)
 {
-    s->beginGroup(colorNameForIndex(index));
-    
+    QString colorName = colorNameForIndex(index);
+
+    s->beginGroup(colorName);
+
     ColorEntry entry;
 
-    QStringList rgbList = s->value("Color", QStringList()).toStringList();
-    if (rgbList.count() != 3)
-    {
-        Q_ASSERT(0);
-    }
+    QVariant colorValue = s->value("Color");
+    QString colorStr;
     int r, g, b;
-    r = rgbList[0].toInt();
-    g = rgbList[1].toInt();
-    b = rgbList[2].toInt();
+    bool ok = false;
+    // XXX: Undocumented(?) QSettings behavior: values with commas are parsed
+    // as QStringList and others QString
+    if (colorValue.type() == QVariant::StringList)
+    {
+        QStringList rgbList = colorValue.toStringList();
+        colorStr = rgbList.join(",");
+        if (rgbList.count() == 3)
+        {
+            bool parse_ok;
+
+            ok = true;
+            r = rgbList[0].toInt(&parse_ok);
+            ok = ok && parse_ok && (r >= 0 && r <= 0xff);
+            g = rgbList[1].toInt(&parse_ok);
+            ok = ok && parse_ok && (g >= 0 && g <= 0xff);
+            b = rgbList[2].toInt(&parse_ok);
+            ok = ok && parse_ok && (b >= 0 && b <= 0xff);
+        }
+    }
+    else
+    {
+        colorStr = colorValue.toString();
+        QRegularExpression hexColorPattern("^#[0-9a-f]{6}$",
+                                           QRegularExpression::CaseInsensitiveOption);
+        if (hexColorPattern.match(colorStr).hasMatch())
+        {
+            // Parsing is always ok as already matched by the regexp
+            r = colorStr.midRef(1, 2).toInt(nullptr, 16);
+            g = colorStr.midRef(3, 2).toInt(nullptr, 16);
+            b = colorStr.midRef(5, 2).toInt(nullptr, 16);
+            ok = true;
+        }
+    }
+    if (!ok)
+    {
+        qWarning().nospace() << "Invalid color value " << colorStr
+                             << " for " << colorName << ". Fallback to black.";
+        r = g = b = 0;
+    }
     entry.color = QColor(r, g, b);
-    
+
     entry.transparent = s->value("Transparent",false).toBool();
 
     // Deprecated key from KDE 4.0 which set 'Bold' to true to force
@@ -362,7 +399,7 @@ void ColorScheme::readColorEntry(QSettings * s , int index)
     setColorTableEntry( index , entry );
 
     if ( hue != 0 || value != 0 || saturation != 0 )
-       setRandomizationRange( index , hue , saturation , value ); 
+       setRandomizationRange( index , hue , saturation , value );
 
     s->endGroup();
 }
@@ -379,8 +416,8 @@ void ColorScheme::writeColorEntry(KConfig& config , const QString& colorName, co
         configGroup.writeEntry("Bold",entry.fontWeight == ColorEntry::Bold);
     }
 
-    // record randomization if this color has randomization or 
-    // if one of the keys already exists 
+    // record randomization if this color has randomization or
+    // if one of the keys already exists
     if ( !random.isNull() || configGroup.hasKey("MaxRandomHue") )
     {
         configGroup.writeEntry("MaxRandomHue",(int)random.hue);
@@ -390,28 +427,28 @@ void ColorScheme::writeColorEntry(KConfig& config , const QString& colorName, co
 }
 #endif
 
-// 
+//
 // Work In Progress - A color scheme for use on KDE setups for users
 // with visual disabilities which means that they may have trouble
 // reading text with the supplied color schemes.
 //
 // This color scheme uses only the 'safe' colors defined by the
-// KColorScheme class.  
+// KColorScheme class.
 //
-// A complication this introduces is that each color provided by 
+// A complication this introduces is that each color provided by
 // KColorScheme is defined as a 'background' or 'foreground' color.
-// Only foreground colors are allowed to be used to render text and 
+// Only foreground colors are allowed to be used to render text and
 // only background colors are allowed to be used for backgrounds.
 //
 // The ColorEntry and TerminalDisplay classes do not currently
-// support this restriction.  
+// support this restriction.
 //
 // Requirements:
 //  - A color scheme which uses only colors from the KColorScheme class
-//  - Ability to restrict which colors the TerminalDisplay widget 
+//  - Ability to restrict which colors the TerminalDisplay widget
 //    uses as foreground and background color
 //  - Make use of KGlobalSettings::allowDefaultBackgroundImages() as
-//    a hint to determine whether this accessible color scheme should 
+//    a hint to determine whether this accessible color scheme should
 //    be used by default.
 //
 //
@@ -444,13 +481,13 @@ AccessibleColorScheme::AccessibleColorScheme()
         colorScheme.foreground( colorScheme.NeutralText )
     };
 
-    for ( int i = 0 ; i < TABLE_COLORS ; i++ ) 
+    for ( int i = 0 ; i < TABLE_COLORS ; i++ )
     {
         ColorEntry entry;
         entry.color = colors[ i % ColorRoleCount ].color();
 
-        setColorTableEntry( i , entry ); 
-    }   
+        setColorTableEntry( i , entry );
+    }
 #endif
 }
 
@@ -458,7 +495,7 @@ KDE3ColorSchemeReader::KDE3ColorSchemeReader( QIODevice* device ) :
     _device(device)
 {
 }
-ColorScheme* KDE3ColorSchemeReader::read() 
+ColorScheme* KDE3ColorSchemeReader::read()
 {
     Q_ASSERT( _device->openMode() == QIODevice::ReadOnly ||
               _device->openMode() == QIODevice::ReadWrite  );
@@ -489,7 +526,7 @@ ColorScheme* KDE3ColorSchemeReader::read()
         {
             qDebug() << "KDE 3 color scheme contains an unsupported feature, '" <<
                 line << "'";
-        } 
+        }
     }
 
     return scheme;
@@ -502,7 +539,7 @@ bool KDE3ColorSchemeReader::readColorLine(const QString& line,ColorScheme* schem
         return false;
     if (list.first() != "color")
         return false;
-    
+
     int index = list[1].toInt();
     int red = list[2].toInt();
     int green = list[3].toInt();
@@ -558,17 +595,13 @@ ColorSchemeManager::~ColorSchemeManager()
 void ColorSchemeManager::loadAllColorSchemes()
 {
     qDebug() << "loadAllColorSchemes";
-    int success = 0;
     int failed = 0;
 
     QList<QString> nativeColorSchemes = listColorSchemes();
-
     QListIterator<QString> nativeIter(nativeColorSchemes);
     while ( nativeIter.hasNext() )
     {
-        if ( loadColorScheme( nativeIter.next() ) )
-            success++;
-        else
+        if ( !loadColorScheme( nativeIter.next() ) )
             failed++;
     }
 
@@ -576,9 +609,7 @@ void ColorSchemeManager::loadAllColorSchemes()
     QListIterator<QString> kde3Iter(kde3ColorSchemes);
     while ( kde3Iter.hasNext() )
     {
-        if ( loadKDE3ColorScheme( kde3Iter.next() ) )
-            success++;
-        else
+        if ( !loadKDE3ColorScheme( kde3Iter.next() ) )
             failed++;
     }
 
@@ -613,7 +644,7 @@ bool ColorSchemeManager::loadKDE3ColorScheme(const QString& filePath)
         delete scheme;
         return false;
     }
-    
+
     QFileInfo info(filePath);
 
     if ( !_colorSchemes.contains(info.baseName()) )
@@ -628,7 +659,7 @@ bool ColorSchemeManager::loadKDE3ColorScheme(const QString& filePath)
     return true;
 }
 #if 0
-void ColorSchemeManager::addColorScheme(ColorScheme* scheme) 
+void ColorSchemeManager::addColorScheme(ColorScheme* scheme)
 {
     _colorSchemes.insert(scheme->name(),scheme);
 
@@ -650,6 +681,11 @@ bool ColorSchemeManager::loadCustomColorScheme(const QString& path)
         return false;
 }
 
+void ColorSchemeManager::addCustomColorSchemeDir(const QString& custom_dir)
+{
+    add_custom_color_scheme_dir(custom_dir);
+}
+
 bool ColorSchemeManager::loadColorScheme(const QString& filePath)
 {
     if ( !filePath.endsWith(QLatin1String(".colorscheme")) || !QFile::exists(filePath) )
@@ -658,17 +694,17 @@ bool ColorSchemeManager::loadColorScheme(const QString& filePath)
     QFileInfo info(filePath);
 
     const QString& schemeName = info.baseName();
-    
+
     ColorScheme* scheme = new ColorScheme();
     scheme->setName(schemeName);
     scheme->read(filePath);
 
-    if (scheme->name().isEmpty()) 
+    if (scheme->name().isEmpty())
     {
         qDebug() << "Color scheme in" << filePath << "does not have a valid name and was not loaded.";
         delete scheme;
         return false;
-    }    
+    }
 
     if ( !_colorSchemes.contains(schemeName) )
     {
@@ -678,23 +714,26 @@ bool ColorSchemeManager::loadColorScheme(const QString& filePath)
     {
         qDebug() << "color scheme with name" << schemeName << "has already been" <<
             "found, ignoring.";
-        
+
         delete scheme;
     }
 
-    return true; 
+    return true;
 }
 QList<QString> ColorSchemeManager::listKDE3ColorSchemes()
 {
-    QString dname(get_color_schemes_dir());
-    QDir dir(dname);
-    QStringList filters;
-    filters << "*.schema";
-    dir.setNameFilters(filters);
-    QStringList list = dir.entryList(filters); 
     QStringList ret;
-    foreach(QString i, list)
-        ret << dname + "/" + i;
+    for (const QString &scheme_dir : get_color_schemes_dirs())
+    {
+        const QString dname(scheme_dir);
+        QDir dir(dname);
+        QStringList filters;
+        filters << "*.schema";
+        dir.setNameFilters(filters);
+        QStringList list = dir.entryList(filters);
+        for (const QString &i : list)
+            ret << dname + "/" + i;
+    }
     return ret;
     //return KGlobal::dirs()->findAllResources("data",
     //                                         "konsole/*.schema",
@@ -703,15 +742,18 @@ QList<QString> ColorSchemeManager::listKDE3ColorSchemes()
 }
 QList<QString> ColorSchemeManager::listColorSchemes()
 {
-    QString dname(get_color_schemes_dir());
-    QDir dir(dname);
-    QStringList filters;
-    filters << "*.colorscheme";
-    dir.setNameFilters(filters);
-    QStringList list = dir.entryList(filters);
     QStringList ret;
-    foreach(QString i, list)
-        ret << dname + "/" + i;
+    for (const QString &scheme_dir : get_color_schemes_dirs())
+    {
+        const QString dname(scheme_dir);
+        QDir dir(dname);
+        QStringList filters;
+        filters << "*.colorscheme";
+        dir.setNameFilters(filters);
+        QStringList list = dir.entryList(filters);
+        for (const QString &i : list)
+            ret << dname + "/" + i;
+    }
     return ret;
 //    return KGlobal::dirs()->findAllResources("data",
 //                                             "konsole/*.colorscheme",
@@ -726,7 +768,7 @@ bool ColorSchemeManager::deleteColorScheme(const QString& name)
 {
     Q_ASSERT( _colorSchemes.contains(name) );
 
-    // lookup the path and delete 
+    // lookup the path and delete
     QString path = findColorSchemePath(name);
     if ( QFile::remove(path) )
     {
@@ -742,16 +784,21 @@ bool ColorSchemeManager::deleteColorScheme(const QString& name)
 QString ColorSchemeManager::findColorSchemePath(const QString& name) const
 {
 //    QString path = KStandardDirs::locate("data","konsole/"+name+".colorscheme");
-    QString path(get_color_schemes_dir() + "/"+ name + ".colorscheme");
+    const QStringList dirs = get_color_schemes_dirs();
+    if ( dirs.isEmpty() )
+        return QString();
+
+    const QString dir = dirs.first();
+    QString path(dir + "/"+ name + ".colorscheme");
     if ( !path.isEmpty() )
-       return path; 
+        return path;
 
     //path = KStandardDirs::locate("data","konsole/"+name+".schema");
-    path = get_color_schemes_dir() + "/"+ name + ".schema";
+    path = dir + "/"+ name + ".schema";
 
     return path;
 }
-const ColorScheme* ColorSchemeManager::findColorScheme(const QString& name) 
+const ColorScheme* ColorSchemeManager::findColorScheme(const QString& name)
 {
     if ( name.isEmpty() )
         return defaultColorScheme();
@@ -761,12 +808,12 @@ const ColorScheme* ColorSchemeManager::findColorScheme(const QString& name)
     else
     {
         // look for this color scheme
-        QString path = findColorSchemePath(name); 
+        QString path = findColorSchemePath(name);
         if ( !path.isEmpty() && loadColorScheme(path) )
         {
-            return findColorScheme(name); 
-        } 
-        else 
+            return findColorScheme(name);
+        }
+        else
         {
             if (!path.isEmpty() && loadKDE3ColorScheme(path))
                 return findColorScheme(name);
@@ -774,15 +821,11 @@ const ColorScheme* ColorSchemeManager::findColorScheme(const QString& name)
 
         qDebug() << "Could not find color scheme - " << name;
 
-        return 0; 
+        return 0;
     }
 }
-
-ColorSchemeManager* ColorSchemeManager::theColorSchemeManager = 0;
-//K_GLOBAL_STATIC( ColorSchemeManager , theColorSchemeManager )
+Q_GLOBAL_STATIC(ColorSchemeManager, theColorSchemeManager)
 ColorSchemeManager* ColorSchemeManager::instance()
 {
-    if (! theColorSchemeManager)
-        theColorSchemeManager = new ColorSchemeManager();
     return theColorSchemeManager;
 }
