@@ -27,7 +27,8 @@
 
 #include <QByteArray>
 
-struct KPtyPrivate {
+class KPtyPrivate {
+public:
 
     Q_DECLARE_PUBLIC(KPty)
 
