@@ -246,8 +246,28 @@ void QTermWidget::init(int startnow)
     m_layout->setMargin(0);
     setLayout(m_layout);
 
+    // translations
+    // First check $XDG_DATA_DIRS. This follows the implementation in libqtxdg
+    QString d = QFile::decodeName(qgetenv("XDG_DATA_DIRS"));
+    QStringList dirs = d.split(QLatin1Char(':'), QString::SkipEmptyParts);
+    if (dirs.isEmpty()) {
+        dirs.append(QString::fromLatin1("/usr/local/share"));
+        dirs.append(QString::fromLatin1("/usr/share"));
+    }
+    dirs.append(QFile::decodeName(TRANSLATIONS_DIR));
+
+    m_translator = new QTranslator(this);
+
+    for (const QString& dir : dirs) {
+        qDebug() << "Trying to load translation file from dir" << dir;
+        if (m_translator->load(QLocale::system(), "qtermwidget", "_", dir)) {
+            qApp->installTranslator(m_translator);
+            qDebug() << "Translations found in" << dir;
+            break;
+        }
+    }
+
     m_impl = new TermWidgetImpl(this);
-    m_impl->m_terminalDisplay->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
     m_layout->addWidget(m_impl->m_terminalDisplay);
 
     connect(m_impl->m_session, SIGNAL(bellRequest(QString)), m_impl->m_terminalDisplay, SLOT(bell(QString)));
@@ -255,6 +275,8 @@ void QTermWidget::init(int startnow)
 
     connect(m_impl->m_session, SIGNAL(activity()), this, SIGNAL(activity()));
     connect(m_impl->m_session, SIGNAL(silence()), this, SIGNAL(silence()));
+    connect(m_impl->m_session, &Session::profileChangeCommandReceived, this, &QTermWidget::profileChanged);
+    connect(m_impl->m_session, &Session::receivedData, this, &QTermWidget::receivedData);
 
     // That's OK, FilterChain's dtor takes care of UrlFilter.
     UrlFilter *urlFilter = new UrlFilter();
@@ -296,13 +318,14 @@ void QTermWidget::init(int startnow)
     m_searchBar->setFont(font);
 
     setScrollBarPosition(NoScrollBar);
-    setKeyboardCursorShape(BlockCursor);
+    setKeyboardCursorShape(Emulation::KeyboardCursorShape::BlockCursor);
 
     m_impl->m_session->addView(m_impl->m_terminalDisplay);
 
     connect(m_impl->m_session, SIGNAL(resizeRequest(QSize)), this, SLOT(setSize(QSize)));
     connect(m_impl->m_session, SIGNAL(finished()), this, SLOT(sessionFinished()));
     connect(m_impl->m_session, &Session::titleChanged, this, &QTermWidget::titleChanged);
+    connect(m_impl->m_session, &Session::cursorChanged, this, &QTermWidget::cursorChanged);
 }
 
 
@@ -668,6 +691,13 @@ void QTermWidget::setKeyboardCursorShape(KeyboardCursorShape shape)
     m_impl->m_terminalDisplay->setKeyboardCursorShape(shape);
 }
 
+void QTermWidget::setBlinkingCursor(bool blink)
+{
+    if (!m_impl->m_terminalDisplay)
+        return;
+    m_impl->m_terminalDisplay->setBlinkingCursor(blink);
+}
+
 QString QTermWidget::title() const
 {
     QString title = m_impl->m_session->userTitle();
@@ -688,3 +718,15 @@ bool QTermWidget::isTitleChanged() const
 {
     return m_impl->m_session->isTitleChanged();
 }
+
+void QTermWidget::setAutoClose(bool autoClose)
+{
+    m_impl->m_session->setAutoClose(autoClose);
+}
+
+void QTermWidget::cursorChanged(Konsole::Emulation::KeyboardCursorShape cursorShape, bool blinkingCursorEnabled)
+{
+    // TODO: A switch to enable/disable DECSCUSR?
+    setKeyboardCursorShape(cursorShape);
+    setBlinkingCursor(blinkingCursorEnabled);
+}
